==> roccat-tools-3.0.0.tar.bz2: roccat-tools-3.0.0/konepureoptical/libkonepureopticaleventhandler/konepureoptical_eventhandler_channel.c <==
/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "konepureoptical_eventhandler_channel.h"
#include "g_cclosure_roccat_marshaller.h"
#include "konepureoptical.h"
#include "roccat.h"
#include "roccat_helper.h"

#define KONEPUREOPTICAL_EVENTHANDLER_CHANNEL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass), KONEPUREOPTICAL_EVENTHANDLER_CHANNEL_TYPE, KonepureopticalEventhandlerChannelClass))
#define IS_KONEPUREOPTICAL_EVENTHANDLER_CHANNEL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), KONEPUREOPTICAL_EVENTHANDLER_CHANNEL_TYPE))
#define KONEPUREOPTICAL_EVENTHANDLER_CHANNEL_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), KONEPUREOPTICAL_EVENTHANDLER_CHANNEL_TYPE, KonepureopticalEventhandlerChannelPrivate))

typedef struct _KonepureopticalEventhandlerChannelClass KonepureopticalEventhandlerChannelClass;
typedef struct _KonepureopticalEventhandlerChannelPrivate KonepureopticalEventhandlerChannelPrivate;

struct _KonepureopticalEventhandlerChannel {
	GObject parent;
	KonepureopticalEventhandlerChannelPrivate *priv;
};

struct _KonepureopticalEventhandlerChannelClass {
	GObjectClass parent_class;
};

struct _KonepureopticalEventhandlerChannelPrivate {
	guint event_source_id;
	gboolean running;
};

enum {
	CPI_CHANGED, /* cpi_index */
	OPEN_DRIVER,
	PROFILE_CHANGED, /* profile_number */
	QUICKLAUNCH, /* key_index */
	SENSITIVITY_CHANGED, /* x, y */
	TALK, /* key_index, event */
	TIMER_START, /* key_index */
	TIMER_STOP,
	LAST_SIGNAL
};

static guint signals[LAST_SIGNAL] = { 0 };

G_DEFINE_TYPE(KonepureopticalEventhandlerChannel, konepureoptical_eventhandler_channel, G_TYPE_OBJECT);

static void process_chrdev_event(KonepureopticalEventhandlerChannel *eventhandler_channel, KoneplusChrdevEvent const *event) {
	gchar *string;

	switch (event->type) {
	case KONEXTD_CHRDEV_EVENT_TYPE_RAD_LEFT:
	case KONEXTD_CHRDEV_EVENT_TYPE_RAD_RIGHT:
	case KONEXTD_CHRDEV_EVENT_TYPE_RAD_MIDDLE:
	case KONEXTD_CHRDEV_EVENT_TYPE_RAD_THUMB_1:
	case KONEXTD_CHRDEV_EVENT_TYPE_RAD_THUMB_2:
	case KONEXTD_CHRDEV_EVENT_TYPE_RAD_SCROLL_WHEEL_UP:
	case KONEXTD_CHRDEV_EVENT_TYPE_RAD_SCROLL_WHEEL_DOWN:
	case KONEXTD_CHRDEV_EVENT_TYPE_RAD_EASYSHIFT:
	case KONEXTD_CHRDEV_EVENT_TYPE_RAD_EASYAIM:
	case KONEXTD_CHRDEV_EVENT_TYPE_RAD_DISTANCE:
		// silently ignore rad events
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_PROFILE:
		g_signal_emit((gpointer)eventhandler_channel, signals[PROFILE_CHANGED], 0, event->data1);
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_CPI:
		g_signal_emit((gpointer)eventhandler_channel, signals[CPI_CHANGED], 0, event->data1 - 1);
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_SENSITIVITY:
		g_signal_emit((gpointer)eventhandler_channel, signals[SENSITIVITY_CHANGED], 0, event->data1, event->data2);
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_QUICKLAUNCH:
		if (event->data2 == KONEPLUS_CHRDEV_EVENT_ACTION_PRESS)
			g_signal_emit((gpointer)eventhandler_channel, signals[QUICKLAUNCH], 0, event->data1 - 1);
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_OPEN_DRIVER:
		g_signal_emit((gpointer)eventhandler_channel, signals[OPEN_DRIVER], 0);
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_TILT:
		/* just ignore this event */
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_TIMER:
		if (event->data2 == KONEPLUS_CHRDEV_EVENT_ACTION_PRESS)
			g_signal_emit((gpointer)eventhandler_channel, signals[TIMER_START], 0, event->data1 - 1);
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_TIMER_STOP:
		if (event->data2 == KONEPLUS_CHRDEV_EVENT_ACTION_PRESS)
			g_signal_emit((gpointer)eventhandler_channel, signals[TIMER_STOP], 0);
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_MULTIMEDIA:
		/* TODO implement multimedia handler */
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_TALK:
		g_signal_emit((gpointer)eventhandler_channel, signals[TALK], 0, event->data1 - 1, event->data2);
		break;
	default:
		string = roccat_data_to_string((guchar const *)event, sizeof(KoneplusChrdevEvent));
		g_debug("unknown event = %s", string);
		g_free(string);
	}
}

static gboolean io_cb(GIOChannel *channel, GIOCondition condition, gpointer user_data) {
	KonepureopticalEventhandlerChannel *eventhandler_channel = KONEPUREOPTICAL_EVENTHANDLER_CHANNEL(user_data);
	KoneplusChrdevEvent event;
	GIOStatus status;
	GError *error = NULL;
	gsize length;

	if (condition & G_IO_HUP)
		return FALSE;

	status = g_io_channel_read_chars(channel, (gchar *)&event, sizeof(KoneplusChrdevEvent), &length, &error);

	if (status == G_IO_STATUS_AGAIN)
		return TRUE;

	if (status == G_IO_STATUS_ERROR || status == G_IO_STATUS_EOF) {
		g_critical("error reading io_channel");
		return FALSE;
	}

	if (length < sizeof(KoneplusChrdevEvent))
		return TRUE;

	switch (event.report_number) {
	case KONEPLUS_CHRDEV_EVENT_REPORT_NUMBER_BUTTON:
		process_chrdev_event(eventhandler_channel, &event);
		break;
	default:
		break;
	}

	return TRUE; // keep on going
}

KonepureopticalEventhandlerChannel *konepureoptical_eventhandler_channel_new(void) {
	return KONEPUREOPTICAL_EVENTHANDLER_CHANNEL(g_object_new(KONEPUREOPTICAL_EVENTHANDLER_CHANNEL_TYPE, NULL));
}

static void konepureoptical_eventhandler_channel_init(KonepureopticalEventhandlerChannel *channel) {
	KonepureopticalEventhandlerChannelPrivate *priv = KONEPUREOPTICAL_EVENTHANDLER_CHANNEL_GET_PRIVATE(channel);
	channel->priv = priv;
	priv->running = FALSE;
}

gboolean konepureoptical_eventhandler_channel_start(KonepureopticalEventhandlerChannel *eventhandler_channel, gchar const *path, GError **error) {
	KonepureopticalEventhandlerChannelPrivate *priv = eventhandler_channel->priv;
	GIOChannel *channel;
	GIOStatus status;

	channel = g_io_channel_new_file(path, "r", error);
	if (!channel)
		return FALSE;

	status = g_io_channel_set_encoding(channel, NULL, error);
	if (status != G_IO_STATUS_NORMAL) {
		g_io_channel_unref(channel);
		return FALSE;
	}

	g_io_channel_set_buffered(channel, FALSE);

	priv->event_source_id = g_io_add_watch(channel, G_IO_IN | G_IO_HUP | G_IO_PRI, (GIOFunc)io_cb, eventhandler_channel);
	/* g_io_add_watch increases reference count
	 * this passes ownership of the channel to the watch */
	g_io_channel_unref(channel);

	return TRUE;
}

void konepureoptical_eventhandler_channel_stop(KonepureopticalEventhandlerChannel *eventhandler_channel) {
	KonepureopticalEventhandlerChannelPrivate *priv = eventhandler_channel->priv;
	if (priv->running) {
		g_source_remove(priv->event_source_id);
		priv->running = FALSE;
	}
}

static void finalize(GObject *object) {
	KonepureopticalEventhandlerChannel *eventhandler_channel = KONEPUREOPTICAL_EVENTHANDLER_CHANNEL(object);
	konepureoptical_eventhandler_channel_stop(eventhandler_channel);
	G_OBJECT_CLASS(konepureoptical_eventhandler_channel_parent_class)->finalize(object);
}

static void konepureoptical_eventhandler_channel_class_init(KonepureopticalEventhandlerChannelClass *klass) {
	GObjectClass *gobject_class;

	gobject_class = (GObjectClass *)klass;

	gobject_class->finalize = finalize;

	g_type_class_add_private(klass, sizeof(KonepureopticalEventhandlerChannelPrivate));

	signals[CPI_CHANGED] = g_signal_new("cpi-changed", KONEPUREOPTICAL_EVENTHANDLER_CHANNEL_TYPE,
			G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			0, NULL, NULL, g_cclosure_marshal_VOID__UCHAR, G_TYPE_NONE,
			1, G_TYPE_UCHAR);

	signals[OPEN_DRIVER] = g_signal_new("open-driver", KONEPUREOPTICAL_EVENTHANDLER_CHANNEL_TYPE,
			G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE,
			0);

	signals[PROFILE_CHANGED] = g_signal_new("profile-changed", KONEPUREOPTICAL_EVENTHANDLER_CHANNEL_TYPE,
			G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			0, NULL, NULL, g_cclosure_marshal_VOID__UCHAR, G_TYPE_NONE,
			1, G_TYPE_UCHAR);

	signals[QUICKLAUNCH] = g_signal_new("quicklaunch", KONEPUREOPTICAL_EVENTHANDLER_CHANNEL_TYPE,
			G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			0, NULL, NULL, g_cclosure_marshal_VOID__UCHAR, G_TYPE_NONE,
			1, G_TYPE_UCHAR);

	signals[SENSITIVITY_CHANGED] = g_signal_new("sensitivity-changed", KONEPUREOPTICAL_EVENTHANDLER_CHANNEL_TYPE,
			G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			0, NULL, NULL, g_cclosure_roccat_marshal_VOID__UCHAR_UCHAR, G_TYPE_NONE,
			2, G_TYPE_UCHAR, G_TYPE_UCHAR);

	signals[TALK] = g_signal_new("talk", KONEPUREOPTICAL_EVENTHANDLER_CHANNEL_TYPE,
			G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			0, NULL, NULL, g_cclosure_roccat_marshal_VOID__UCHAR_UCHAR, G_TYPE_NONE,
			2, G_TYPE_UCHAR, G_TYPE_UCHAR);

	signals[TIMER_START] = g_signal_new("timer-start", KONEPUREOPTICAL_EVENTHANDLER_CHANNEL_TYPE,
			G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			0, NULL, NULL, g_cclosure_marshal_VOID__UCHAR, G_TYPE_NONE,
			1, G_TYPE_UCHAR);

	signals[TIMER_STOP] = g_signal_new("timer-stop", KONEPUREOPTICAL_EVENTHANDLER_CHANNEL_TYPE,
			G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE,
			0);
}

==> roccat-tools-3.4.0.tar.bz2: roccat-tools-3.4.0/konepureoptical/libkonepureopticaleventhandler/konepureoptical_eventhandler_channel.c <==
/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "konepureoptical_eventhandler_channel.h"
#include "g_cclosure_roccat_marshaller.h"
#include "konepureoptical.h"
#include "roccat.h"
#include "roccat_helper.h"
#include "i18n-lib.h"

#define KONEPUREOPTICAL_EVENTHANDLER_CHANNEL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass), KONEPUREOPTICAL_EVENTHANDLER_CHANNEL_TYPE, KonepureopticalEventhandlerChannelClass))
#define IS_KONEPUREOPTICAL_EVENTHANDLER_CHANNEL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), KONEPUREOPTICAL_EVENTHANDLER_CHANNEL_TYPE))
#define KONEPUREOPTICAL_EVENTHANDLER_CHANNEL_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), KONEPUREOPTICAL_EVENTHANDLER_CHANNEL_TYPE, KonepureopticalEventhandlerChannelPrivate))

typedef struct _KonepureopticalEventhandlerChannelClass KonepureopticalEventhandlerChannelClass;
typedef struct _KonepureopticalEventhandlerChannelPrivate KonepureopticalEventhandlerChannelPrivate;

struct _KonepureopticalEventhandlerChannel {
	GObject parent;
	KonepureopticalEventhandlerChannelPrivate *priv;
};

struct _KonepureopticalEventhandlerChannelClass {
	GObjectClass parent_class;
};

struct _KonepureopticalEventhandlerChannelPrivate {
	guint event_source_id;
	gboolean running;
};

enum {
	CPI_CHANGED, /* cpi_index */
	OPEN_DRIVER,
	PROFILE_CHANGED, /* profile_number */
	QUICKLAUNCH, /* key_index */
	SENSITIVITY_CHANGED, /* x, y */
	TALK, /* key_index, event */
	TIMER_START, /* key_index */
	TIMER_STOP,
	LAST_SIGNAL
};

static guint signals[LAST_SIGNAL] = { 0 };

G_DEFINE_TYPE(KonepureopticalEventhandlerChannel, konepureoptical_eventhandler_channel, G_TYPE_OBJECT);

static void process_chrdev_event(KonepureopticalEventhandlerChannel *eventhandler_channel, KoneplusChrdevEvent const *event) {
	gchar *string;

	switch (event->type) {
	case KONEXTD_CHRDEV_EVENT_TYPE_RAD_LEFT:
	case KONEXTD_CHRDEV_EVENT_TYPE_RAD_RIGHT:
	case KONEXTD_CHRDEV_EVENT_TYPE_RAD_MIDDLE:
	case KONEXTD_CHRDEV_EVENT_TYPE_RAD_THUMB_1:
	case KONEXTD_CHRDEV_EVENT_TYPE_RAD_THUMB_2:
	case KONEXTD_CHRDEV_EVENT_TYPE_RAD_SCROLL_WHEEL_UP:
	case KONEXTD_CHRDEV_EVENT_TYPE_RAD_SCROLL_WHEEL_DOWN:
	case KONEXTD_CHRDEV_EVENT_TYPE_RAD_EASYSHIFT:
	case KONEXTD_CHRDEV_EVENT_TYPE_RAD_EASYAIM:
	case KONEXTD_CHRDEV_EVENT_TYPE_RAD_DISTANCE:
		// silently ignore rad events
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_PROFILE:
		g_signal_emit((gpointer)eventhandler_channel, signals[PROFILE_CHANGED], 0, event->data1);
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_CPI:
		g_signal_emit((gpointer)eventhandler_channel, signals[CPI_CHANGED], 0, event->data1 - 1);
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_SENSITIVITY:
		g_signal_emit((gpointer)eventhandler_channel, signals[SENSITIVITY_CHANGED], 0, event->data1, event->data2);
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_QUICKLAUNCH:
		if (event->data2 == KONEPLUS_CHRDEV_EVENT_ACTION_PRESS)
			g_signal_emit((gpointer)eventhandler_channel, signals[QUICKLAUNCH], 0, event->data1 - 1);
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_OPEN_DRIVER:
		g_signal_emit((gpointer)eventhandler_channel, signals[OPEN_DRIVER], 0);
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_TILT:
		/* just ignore this event */
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_TIMER:
		if (event->data2 == KONEPLUS_CHRDEV_EVENT_ACTION_PRESS)
			g_signal_emit((gpointer)eventhandler_channel, signals[TIMER_START], 0, event->data1 - 1);
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_TIMER_STOP:
		if (event->data2 == KONEPLUS_CHRDEV_EVENT_ACTION_PRESS)
			g_signal_emit((gpointer)eventhandler_channel, signals[TIMER_STOP], 0);
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_MULTIMEDIA:
		/* TODO implement multimedia handler */
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_TALK:
		g_signal_emit((gpointer)eventhandler_channel, signals[TALK], 0, event->data1 - 1, event->data2);
		break;
	default:
		string = roccat_data_to_string((guchar const *)event, sizeof(KoneplusChrdevEvent));
		g_debug(_("Got unknown event: %s"), string);
		g_free(string);
	}
}

static gboolean io_cb(GIOChannel *channel, GIOCondition condition, gpointer user_data) {
	KonepureopticalEventhandlerChannel *eventhandler_channel = KONEPUREOPTICAL_EVENTHANDLER_CHANNEL(user_data);
	KoneplusChrdevEvent event;
	GIOStatus status;
	GError *error = NULL;
	gsize length;

	if (condition & G_IO_HUP)
		return FALSE;

	status = g_io_channel_read_chars(channel, (gchar *)&event, sizeof(KoneplusChrdevEvent), &length, &error);

	if (status == G_IO_STATUS_AGAIN)
		return TRUE;

	if (status == G_IO_STATUS_ERROR) {
		g_critical(_("Could not read io_channel: %s"), error->message);
		return FALSE;
	}

	if (status == G_IO_STATUS_EOF) {
		g_warning(_("Could not read io_channel: %s"), _("End of file"));
		return FALSE;
	}

	if (length < sizeof(KoneplusChrdevEvent))
		return TRUE;

	switch (event.report_number) {
	case KONEPLUS_CHRDEV_EVENT_REPORT_NUMBER_BUTTON:
		process_chrdev_event(eventhandler_channel, &event);
		break;
	default:
		break;
	}

	return TRUE; // keep on going
}

KonepureopticalEventhandlerChannel *konepureoptical_eventhandler_channel_new(void) {
	return KONEPUREOPTICAL_EVENTHANDLER_CHANNEL(g_object_new(KONEPUREOPTICAL_EVENTHANDLER_CHANNEL_TYPE, NULL));
}

static void konepureoptical_eventhandler_channel_init(KonepureopticalEventhandlerChannel *channel) {
	KonepureopticalEventhandlerChannelPrivate *priv = KONEPUREOPTICAL_EVENTHANDLER_CHANNEL_GET_PRIVATE(channel);
	channel->priv = priv;
	priv->running = FALSE;
}

gboolean konepureoptical_eventhandler_channel_start(KonepureopticalEventhandlerChannel *eventhandler_channel, gchar const *path, GError **error) {
	KonepureopticalEventhandlerChannelPrivate *priv = eventhandler_channel->priv;
	GIOChannel *channel;
	GIOStatus status;

	channel = g_io_channel_new_file(path, "r", error);
	if (!channel)
		return FALSE;

	status = g_io_channel_set_encoding(channel, NULL, error);
	if (status != G_IO_STATUS_NORMAL) {
		g_io_channel_unref(channel);
		return FALSE;
	}

	g_io_channel_set_buffered(channel, FALSE);

	priv->event_source_id = g_io_add_watch(channel, G_IO_IN | G_IO_HUP | G_IO_PRI, (GIOFunc)io_cb, eventhandler_channel);
	/* g_io_add_watch increases reference count
	 * this passes ownership of the channel to the watch */
	g_io_channel_unref(channel);

	return TRUE;
}

void konepureoptical_eventhandler_channel_stop(KonepureopticalEventhandlerChannel *eventhandler_channel) {
	KonepureopticalEventhandlerChannelPrivate *priv = eventhandler_channel->priv;
	if (priv->running) {
		g_source_remove(priv->event_source_id);
		priv->running = FALSE;
	}
}

static void finalize(GObject *object) {
	KonepureopticalEventhandlerChannel *eventhandler_channel = KONEPUREOPTICAL_EVENTHANDLER_CHANNEL(object);
	konepureoptical_eventhandler_channel_stop(eventhandler_channel);
	G_OBJECT_CLASS(konepureoptical_eventhandler_channel_parent_class)->finalize(object);
}

static void konepureoptical_eventhandler_channel_class_init(KonepureopticalEventhandlerChannelClass *klass) {
	GObjectClass *gobject_class;

	gobject_class = (GObjectClass *)klass;

	gobject_class->finalize = finalize;

	g_type_class_add_private(klass, sizeof(KonepureopticalEventhandlerChannelPrivate));

	signals[CPI_CHANGED] = g_signal_new("cpi-changed", KONEPUREOPTICAL_EVENTHANDLER_CHANNEL_TYPE,
			G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			0, NULL, NULL, g_cclosure_marshal_VOID__UCHAR, G_TYPE_NONE,
			1, G_TYPE_UCHAR);

	signals[OPEN_DRIVER] = g_signal_new("open-driver", KONEPUREOPTICAL_EVENTHANDLER_CHANNEL_TYPE,
			G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE,
			0);

	signals[PROFILE_CHANGED] = g_signal_new("profile-changed", KONEPUREOPTICAL_EVENTHANDLER_CHANNEL_TYPE,
			G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			0, NULL, NULL, g_cclosure_marshal_VOID__UCHAR, G_TYPE_NONE,
			1, G_TYPE_UCHAR);

	signals[QUICKLAUNCH] = g_signal_new("quicklaunch", KONEPUREOPTICAL_EVENTHANDLER_CHANNEL_TYPE,
			G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			0, NULL, NULL, g_cclosure_marshal_VOID__UCHAR, G_TYPE_NONE,
			1, G_TYPE_UCHAR);

	signals[SENSITIVITY_CHANGED] = g_signal_new("sensitivity-changed", KONEPUREOPTICAL_EVENTHANDLER_CHANNEL_TYPE,
			G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			0, NULL, NULL, g_cclosure_roccat_marshal_VOID__UCHAR_UCHAR, G_TYPE_NONE,
			2, G_TYPE_UCHAR, G_TYPE_UCHAR);

	signals[TALK] = g_signal_new("talk", KONEPUREOPTICAL_EVENTHANDLER_CHANNEL_TYPE,
			G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			0, NULL, NULL, g_cclosure_roccat_marshal_VOID__UCHAR_UCHAR, G_TYPE_NONE,
			2, G_TYPE_UCHAR, G_TYPE_UCHAR);

	signals[TIMER_START] = g_signal_new("timer-start", KONEPUREOPTICAL_EVENTHANDLER_CHANNEL_TYPE,
			G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			0, NULL, NULL, g_cclosure_marshal_VOID__UCHAR, G_TYPE_NONE,
			1, G_TYPE_UCHAR);

	signals[TIMER_STOP] = g_signal_new("timer-stop", KONEPUREOPTICAL_EVENTHANDLER_CHANNEL_TYPE,
			G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE,
			0);
}
